#!/usr/bin/env python

import os, sys
from optparse import OptionParser
from freemet.config import config

try:
    import wx
    _ = wx.GetTranslation
except:
    print "Error loading wx"
    sys.exit(1)
    
class FreeMetApp(wx.App):
    """
    The main application class where everything begins
    """
    def OnInit(self):
        basepath = os.path.abspath(os.path.dirname(sys.argv[0]))
        localedir = os.path.join(basepath, "locale")
        
        lang = ""
        langid = wx.LANGUAGE_DEFAULT
        
        self.locale = wx.Locale(langid)
        self.locale.AddCatalogLookupPathPrefix(localedir)
        
        if hasattr(sys, 'frozen'):
            self.locale.AddCatalog('wxstd')
        catFound = self.locale.AddCatalog('freemet')
        
        # Import freemet modules after i18n is initialized
        import freemet.splash as fmsplash
        import freemet.frame as fmframe
        
        # Create splash screen
        #splash = fmsplash.MySplashScreen()
        
        # Create frame to start loading weather data
        frame = fmframe.Frame()
        
        #splash.Close()
        
        if config["iconize"]:
            frame.Hide()
        else:
            frame.Show()
        
        return True

if __name__ == '__main__':
    try:
        assert sys.argv[0]
        
        path = os.path.abspath(os.path.dirname(sys.argv[0]))
        os.chdir(path)
        
    except:
        pass
    
    # Parse command line arguments
    parser = OptionParser()
    parser.add_option("-v", "--verbose", action="store_true",
                      help=_("Shows debugging messages during runtime"))
    parser.add_option("-i", "--iconize", action="store_true",
                      help=_("Iconizes FreeMet after startup"))
    parser.add_option("-d", "--disable", action="append",
                      help=_("Disables the following functions:" \
                             " download (do not download anything)," \
                             " tbicon (do not create taskbar icon)"))
    options, args = parser.parse_args()

    # Handle command line arguments
    # -ddownload
    config["disable_download"] = False
    config["disable_tbicon"] = False
    if options.disable:
        for disable in options.disable:
            disablestring = "disable_" + disable
            config[disablestring] = True
    # -v / --verbose
    config["verbose"] = options.verbose
    # -i / --iconize
    config["iconize"] = options.iconize
    
    # Check wx version
    version = map(lambda x: int(x), wx.__version__.split("."))
    if version[0] >= 2 and version[1] >= 8 and version[2] >= 8:
        # Launch application
        app = FreeMetApp()    
        app.SetAppName("FreeMet")
        app.MainLoop()
    else:
        print "Your wx installation is outdated, please use " \
              "2.8.8.1 or later."
        sys.exit(1)
