import wx
import os, shutil
from freemet.config import config

_ = wx.GetTranslation

class ServiceSettingsDialog(wx.Dialog):
    def __init__(self, parent, dictServices, listLoadOnStartup, setname):
        wx.Dialog.__init__(self, parent, -1, title=_("Service Settings"))
        
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        
        self.dictServices = dictServices
        self.listLoadOnStartup = listLoadOnStartup
        
        # Sizers
        mainsizer = wx.BoxSizer(wx.VERTICAL)
        contentsizer = wx.FlexGridSizer(2, 2)
        
        # Description
        mainsizer.Add(wx.StaticText(self, -1, _("Current set is: ") + setname))
        
        # Reverse lookup table for service names
        self.reverseNameLookup = {}
        for key, obj in self.dictServices.iteritems():
            sName, fName = obj.GetNameTuple()
            self.reverseNameLookup[fName] = key
        
        # Service selection box
        serviceNames = self.reverseNameLookup.keys()
        serviceNames.sort()
        self.comboboxService = wx.ComboBox(self, -1, serviceNames[0],
                                           style=wx.CB_READONLY,
                                           choices=serviceNames)
        mainsizer.Add(self.comboboxService)
        self.Bind(wx.EVT_COMBOBOX, self.OnComboBox, self.comboboxService)
        
        # Update interval
        firstServiceKey = self.reverseNameLookup[serviceNames[0]]
        firstService = self.dictServices[firstServiceKey]
        updateInterval = firstService.GetUpdateInterval()
        contentsizer.Add(wx.StaticText(self, -1, _("Update inverval:")),
                         0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        self.spinUpdateInterval = wx.SpinCtrl(self, -1, size=(40, -1),
                                              min=0, max=9999,
                                              initial=updateInterval)
        self.Bind(wx.EVT_SPINCTRL, self.OnSpinCtrl, self.spinUpdateInterval)
        contentsizer.Add(self.spinUpdateInterval)
        
        # Load-on-startup checkbox
        loadOnStartup = firstServiceKey in self.listLoadOnStartup
        contentsizer.Add(wx.StaticText(self, -1, _("Load on startup:")),
                         0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        self.checkboxLoadOnStartup = wx.CheckBox(self, -1)
        self.checkboxLoadOnStartup.SetValue(loadOnStartup)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self.checkboxLoadOnStartup)
        contentsizer.Add(self.checkboxLoadOnStartup)
        
        # Add content to main sizer
        mainsizer.Add(contentsizer, 0, wx.ALL, 10)
        
        # Close button
        btnClose = wx.Button(self, wx.ID_OK)
        assert btnClose
        mainsizer.Add(btnClose, 0, wx.ALL | wx.ALIGN_RIGHT, 10)
        self.Bind(wx.EVT_BUTTON, self.OnClose, btnClose)
        
        # Put everything together
        self.SetSizer(mainsizer)
        self.Fit()
        
    def OnClose(self, evt):
        self.EndModal(wx.ID_CANCEL)
        
    def OnComboBox(self, evt):
        name = self.comboboxService.GetValue()
        key = self.reverseNameLookup[name]
        
        updateInterval = self.dictServices[key].GetUpdateInterval()
        self.spinUpdateInterval.SetValue(updateInterval)
        
        loadOnStartup = key in self.listLoadOnStartup
        self.checkboxLoadOnStartup.SetValue(loadOnStartup)
    
    def OnSpinCtrl(self, evt):
        name = self.comboboxService.GetValue()
        key = self.reverseNameLookup[name]
        i = self.spinUpdateInterval.GetValue()
        
        self.dictServices[key].SetUpdateInterval(i)
        
    def OnCheckBox(self, evt):
        name = self.comboboxService.GetValue()
        key = self.reverseNameLookup[name]
        
        # Add service keyname to startup list
        if self.checkboxLoadOnStartup.GetValue():
            if key not in self.listLoadOnStartup:
                self.listLoadOnStartup.append(key)
                
        # Remove service keyname from startup list
        else:
            if key in self.listLoadOnStartup:
                self.listLoadOnStartup.remove(key)