from freemet.services.service import Service
import wx
import re

_ = wx.GetTranslation

baseurl = "http://www.wetterzentrale.de/pics/"
preurl_ground = "MS_"
preurl_atmo = "MU_"
preurl_eur = "MS_" 
extension_grid = "_g05.png"
extension_city = "_avn.png"
extension_eur = "eur_g05.png"

middleEuropeLatitudes = (43, 57)
middleEuropeLongitudes = (0, 20)
europeLatitudes = (30, 72)
europeLongitudes = (-25, 40)

cities = ["Aberdeen","Ajaccio","Akureyri","Alicante","Almeria","Ankara",
          "Antalya","Archangelsk","Athen","Barcelona","Belgrad","Bergen",
          "Berlin","Bordeaux","Brest","Bruessel","Budapest","Bukarest",
          "De Bilt","Dresden","Dublin","Essen","Frankfurt Main","Funchal",
          "Genf","Gibraltar","Hamburg","Haparanda","Helgoland","Helsinki",
          "Istanbul","Jan Mayen","Karlsruhe","Kiel","Kiew","Kopenhagen",
          "La Coruna","La Palma","Larnaca","Lissabon","London","Luqa",
          "Luxemburg","Madrid","Manchester","Marseille","Moskau","Muenchen",
          "Nuuk","Odessa","Oslo","Paris","Porto","Prag","Reykjavik","Rom",
          "Sodankylae","Sofia","St.Petersburg","Stockholm","Strasbourg",
          "Stuttgart","Thorshavn","Toulouse","Tromsoe","Verona","Warschau",
          "Wien"]

# Coordinates of gfs cities (lat, lon), whole-number steps only
coords = { "Aberdeen": (57, -2), "Ajaccio": (42, 9),
           "Akureyri": (66, -18), "Alicante": (38, 0),
           "Almeria": (37, -2), "Ankara": (40, 33),
           "Antalya": (37, 31), "Archangelsk": (65, 41),
           "Athen": (38, 24), "Barcelona": (41, 2),
           "Belgrad": (45, 20), "Bergen": (60, 5),
           "Berlin": (52.5, 13.5), "Bordeaux": (45, -1),
           "Brest": (48, -4), "Bruessel": (51, 4.5),
           "Budapest": (47.5, 19), "Bukarest": (45, 26),
           "De Bilt": (52, 5), "Dresden": (51, 14),
           "Dublin": (53, -6), "Essen": (51.5, 7),
           "Frankfurt Main": (50, 9), "Funchal": (33, -17),
           "Genf": (46, 6), "Gibraltar": (36, -5),
           "Hamburg": (53.5, 10), "Haparanda": (66, 24),
           "Helgoland": (54, 8), "Helsinki": (60, 25),
           "Istanbul": (41, 29), "Jan Mayen": (71, -9),
           "Karlsruhe": (49, 8.5), "Kiel": (54.5, 10),
           "Kiew": (50, 30), "Kopenhagen": (55.5, 12.5),
           "La Coruna": (43, -8), "La Palma": (29, -18),
           "Larnaca": (35, 34), "Lissabon": (39, -9),
           "London": (51.5, 0), "Luqa": (36, 14),
           "Luxemburg": (49.5, 6), "Madrid": (40, -4),
           "Manchester": (53, -2), "Marseille": (43.5, 5.5),
           "Moskau": (56, 38), "Muenchen": (48, 11.5),
           "Nuuk": (64, -52), "Odessa": (46, 31),
           "Oslo": (60, 11), "Paris": (49, 2.5),
           "Porto": (41, -9), "Prag": (50, 14.5),
           "Reykjavik": (64, -22), "Rom": (42, 12),
           "Sodankylae": (67, 27), "Sofia": (42.5, 23.5),
           "St.Petersburg": (60, 30), "Stockholm": (59, 18),
           "Strasbourg": (48.5, 8), "Stuttgart": (48.5, 9),
           "Thorshavn": (62, -7), "Toulouse": (43.5, 1.5),
           "Tromsoe": (70, 19), "Verona": (45.5, 11),
           "Warschau": (52, 21), "Wien": (48, 16)}

pages = [_("Ground"), _("Atmosphere"), _("City"), _("Settings"), _("About")]

class DegreeValidator(wx.PyValidator):
    regexp = re.compile("^-?[0-9]{1,3}(\.[05])?$")
    
    def __init__(self, minDegree, maxDegree):
        wx.PyValidator.__init__(self)
        
        self.valid = True
        
        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        
        self.minDegree = minDegree
        self.maxDegree = maxDegree

    def Clone(self):
        return DegreeValidator(self.minDegree, self.maxDegree)
    
    def Reset(self):
        self.__Unbell()
        self.valid = True
    
    def OnKeyUp(self, evt):
        key = evt.GetKeyCode()
        ctrl = self.GetWindow()
        text = ctrl.GetValue()

        # Quit (don't check regexp) if key is not backspace, delete, the decimal
        # point, the hyphen, or a digit
        if key != wx.WXK_BACK and key != wx.WXK_DELETE:
            if key >= wx.WXK_SPACE and key <= 255:
                if chr(key) not in "0123456789-.":
                    return
            else:
                return
            
        # Check for regular expression
        if not self.regexp.match(text):
            self.__Bell()
            self.valid = False
            
        # Even if the regular expression is valid, the degree might be invalid
        # (out of range)
        else:
            degree = float(text)
            if degree < self.minDegree or degree > self.maxDegree:
                self.__Bell()
                self.valid = False
            else:
                self.__Unbell()
                self.valid = True

    def OnChar(self, evt):
        key = evt.GetKeyCode()
        allowed = "0123456789-."
        ctrl = self.GetWindow()
        text = ctrl.GetValue()
        
        if key >= wx.WXK_SPACE and key != wx.WXK_DELETE and key <= 255:
            if chr(key) not in allowed:
                self.__Blink()
                return

        evt.Skip()
        self.__Unbell()
        
    def __Bell(self):
        ctrl = self.GetWindow()
        ctrl.SetBackgroundColour("pink")
        ctrl.SetFocus()
        ctrl.Refresh()
        
    def __Unbell(self):
        ctrl = self.GetWindow()
        ctrl.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
        ctrl.Refresh()
        
    def __Blink(self):
        self.__Bell()
        wx.CallLater(250, self.__Unbell)
       
class PropertiesPanel(wx.Panel):
    def __init__(self, parent, service, values):
        wx.Panel.__init__(self, parent, -1)
        
        self.service = service        
        self.origValues = values
        self.values = dict(values)
        self.SPINBTN_LONGITUDE = -1
        self.SPINBTN_LATITUDE = -1
        
        # Create sizers
        mainsizer   = wx.BoxSizer(wx.VERTICAL)
        citysizer   = wx.BoxSizer(wx.HORIZONTAL)
        degreesizer = wx.FlexGridSizer(2, 4)
        buttonsizer = wx.BoxSizer(wx.HORIZONTAL)
        
        def CreateBoundSpinButtons(range, value):
            id = wx.NewId()
            
            ctrl = wx.SpinButton(self, id)
            ctrl.SetRange(range[0] * 2, range[1] * 2)
            ctrl.SetValue(value * 2)
            
            self.Bind(wx.EVT_SPIN, self.OnSpinButton, ctrl)
            
            return ctrl, id
        
        # City dropdown menu
        citysizer.Add(wx.StaticText(self, -1, _("City:")), 0,
                        wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        self.comboboxCity = wx.ComboBox(self, -1, style=wx.CB_READONLY,
                                        value=self.values["city"],
                                        choices=cities)
        citysizer.Add(self.comboboxCity)
        self.Bind(wx.EVT_COMBOBOX, self.OnCombobox, self.comboboxCity)
        
        # Create text controls and spin buttons for longitude and latitude
        # Latitude (static text, text ctrl, spin buttons)
        degreesizer.Add(wx.StaticText(self, -1, _("Latitude:")), 0,
                        wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        
        self.textctrlLatitude = wx.TextCtrl(self, -1,
                                            value=str(self.values["latitude"]),
                                            validator=DegreeValidator(europeLatitudes[0],
                                                                      europeLatitudes[1]))
        self.Bind(wx.EVT_TEXT, self.OnTextCtrl, self.textctrlLatitude)
        degreesizer.Add(self.textctrlLatitude, 1)
        sb, self.SPINBTN_LATITUDE = CreateBoundSpinButtons(europeLatitudes,
                                                           self.values["latitude"])
        degreesizer.Add(sb, 0, wx.ALIGN_CENTER_VERTICAL)
        degreesizer.Add(wx.StaticText(self, -1, _(u"\xb0N")), 0,
                        wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        
        # Longitude (static text, text ctrl, spin buttons)
        degreesizer.Add(wx.StaticText(self, -1, _("Longitude:")), 0,
                        wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        
        self.textctrlLongitude = wx.TextCtrl(self, -1,
                                             value=str(self.values["longitude"]),
                                             validator=DegreeValidator(europeLongitudes[0],
                                                                       europeLongitudes[1]))
        self.Bind(wx.EVT_TEXT, self.OnTextCtrl, self.textctrlLongitude)
        degreesizer.Add(self.textctrlLongitude, 1)
        sb, self.SPINBTN_LONGITUDE = CreateBoundSpinButtons(europeLongitudes,
                                                            self.values["longitude"])
        degreesizer.Add(sb, 0, wx.ALIGN_CENTER_VERTICAL)
        degreesizer.Add(wx.StaticText(self, -1, _(u"\xb0E")), 0,
                        wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        
        # Create undo and apply buttons
        for btnId in [wx.ID_UNDO, wx.ID_APPLY]:
            btn = wx.Button(self, btnId)
            btn.Disable()
            self.Bind(wx.EVT_BUTTON, self.OnButton, btn)   
            buttonsizer.Add(btn, 0, wx.ALIGN_BOTTOM | wx.ALL, 5)
        
        # Put everything together
        mainsizer.Add(citysizer, 0, wx.ALL, 5)
        mainsizer.Add(degreesizer, 0, wx.ALL, 5)
        mainsizer.Add(buttonsizer, 1, wx.ALIGN_BOTTOM | wx.ALIGN_RIGHT | wx.ALL, 5)
        self.SetSizer(mainsizer)
        
    def OnSpinButton(self, evt):
        id = evt.GetId()
        
        if id == self.SPINBTN_LONGITUDE: ctrl = self.textctrlLongitude
        elif id == self.SPINBTN_LATITUDE: ctrl = self.textctrlLatitude
        
        # Divide position by 2 because values have step of 0.5
        ctrl.SetValue(str(evt.GetPosition() * 0.5))
        ctrl.GetValidator().Reset()
    
    def OnTextCtrl(self, evt):
        id = evt.GetId()
        
        if id == self.textctrlLongitude.GetId():
            ctrl = self.textctrlLongitude
            sb = self.FindWindowById(self.SPINBTN_LONGITUDE)
        elif id == self.textctrlLatitude.GetId():
            ctrl = self.textctrlLatitude
            sb = self.FindWindowById(self.SPINBTN_LATITUDE)
        
        # Try to set spin buttons    
        text = ctrl.GetValue()
        try:
            value = float(text)
        except:
            pass
        else:
            sb.SetValue(2 * value)
            
        # Activate buttons
        for id in wx.ID_APPLY, wx.ID_UNDO:
            btn = self.FindWindowById(id)
            btn.Enable()
            
    def OnCombobox(self, evt):
        city = self.comboboxCity.GetValue()
        self.values["city"] = city
        
        # Set longitude and latitude text fields
        lat, lon = coords[city]
        self.textctrlLongitude.SetValue(str(lon))
        self.textctrlLatitude.SetValue(str(lat))
        
        # Activate buttons
        for id in wx.ID_APPLY, wx.ID_UNDO:
            btn = self.FindWindowById(id)
            btn.Enable()
        
    def OnButton(self, evt):
        id = evt.GetId()
        
        if id == wx.ID_APPLY:
            # Check for correct longitude and latitude values first
            valLon = self.textctrlLongitude.GetValidator()
            valLat = self.textctrlLatitude.GetValidator()
            
            if not valLon.valid:
                wx.MessageBox(_("Invalid longitude value"), _("Error"),
                              wx.OK | wx.ICON_ERROR)
                return
            
            elif not valLat.valid:
                wx.MessageBox(_("Invalid latitude value"), _("Error"),
                              wx.OK | wx.ICON_ERROR)
                return
            
            self.values["longitude"] = float(self.textctrlLongitude.GetValue())
            self.values["latitude"] = float(self.textctrlLatitude.GetValue())
            self.values["city"] = self.comboboxCity.GetValue()
            
            #for x, y in self.values.iteritems():
            #    self.origValues[x] = y
            self.origValues = dict(self.values)
                
            self.service.OnPropertiesApply(self.values)
            
        elif id == wx.ID_UNDO:
            self.values = dict(self.origValues)
            self.textctrlLongitude.SetValue(str(self.values["longitude"]))
            self.textctrlLatitude.SetValue(str(self.values["latitude"]))
            self.comboboxCity.SetValue(self.values["city"])
            
            # Disable buttons
            for id in wx.ID_APPLY, wx.ID_UNDO:
                btn = self.FindWindowById(id)
                btn.Disable()
                
            # Set validators to valid=True
            for ctrl in self.textctrlLongitude, self.textctrlLatitude:
                ctrl.GetValidator().Reset()
        
class GfsWeekForecast(Service):
    def __init__(self):
        Service.__init__(self, "gfsweekforecast", _("GFS Week Forecast"), 3, 60,
                         pages, "forecast")
        self.images = None
        
        # Default values
        self.longitude = 8.5
        self.latitude = 49.0
        self.city = "Karlsruhe"

        cnfLongitude = self.ReadConfigFloat("longitude")
        cnfLatitude = self.ReadConfigFloat("latitude")
        self.city = self.ReadConfigString("city")
        
        if cnfLongitude > 0:
            self.longitude = cnfLongitude
        if cnfLatitude > 0:
            self.latitude = cnfLatitude
        
    def Load(self, download, progressDlg=None, progressCount=0):
        # Check if coords lie in Middle Europe
        if self.longitude >= middleEuropeLongitudes[0] and \
           self.longitude <= middleEuropeLongitudes[1] and \
           self.latitude >= middleEuropeLatitudes[0] and \
           self.latitude <= middleEuropeLatitudes[1]:
            # Preceed coord number with one leading zero if less than 10 and
            # remove decimal point
            longitude = "%04.1f" % self.longitude
            latitude = "%04.1f" % self.latitude
            longitude = longitude.replace(".", "")
            latitude = latitude.replace(".", "")
            mapUrl = baseurl + preurl_ground + longitude + latitude + extension_grid
        else:
            # Convert longitude and latitude integers to strings
            longitude = "%i" % self.longitude
            latitude = "%i" % self.latitude
            mapUrl = baseurl + preurl_eur + longitude + latitude + extension_eur 
            
        city = self.city.replace(" ", "_")
            
        sName, fName = self.GetNameTuple()
        downloads = [
            (mapUrl,
             sName + "_ground.png"),
            (baseurl + preurl_atmo + city + extension_city,
             sName + "_atmo.png"),
            (baseurl + preurl_ground + city + extension_city,
             sName + "_city.png")]
        
        return self.Download(download, downloads, progressDlg, progressCount)
    
    def CreatePage(self, index):
        if self.images == None: self.images = []
        
        sName, fName = self.GetNameTuple()
        
        if index == 0:
            p = wx.Panel(Service.notebook, -1)
            img = self.AddImageNotebookPage(p, sName + "_ground.png")
            self.images.append(img)
        
        elif index == 1:
            p = wx.Panel(Service.notebook, -1)
            img = self.AddImageNotebookPage(p, sName + "_atmo.png")
            self.images.append(img)
        
        elif index == 2:
            p = wx.Panel(Service.notebook, -1)
            img = self.AddImageNotebookPage(p, sName + "_city.png")
            self.images.append(img)
        
        elif index == 3:
            values = {"longitude": self.longitude,
                      "latitude": self.latitude,
                      "city": self.city}
            p = PropertiesPanel(Service.notebook, self, values)
            self.propertiesPanel = p
        
        elif index == 4:
            descr = _("This service downloads the GFS one-week forecast from\n"\
                      "www.wetterzentrale.de depending on the coordinates and\n"\
                      "the city the user entered in the settings.")
            p = self.CreateAboutPage(descr, "(c) 2008 Tobias Wulff", "GPL 2")
        
        return p
    
    def ActivateNotebookPage(self, index):
        pass
            
    def DeactivateNotebookPage(self, index):
        pass
    
    def OnPropertiesApply(self, values):
        self.longitude = values["longitude"]
        self.latitude = values["latitude"]
        self.city = values["city"]
        
        # Write new settings to config file
        self.__WriteConfigFile()
        
        # Reload all images
        self.Load(True)
        self.RefreshPages(True)
    
    def __WriteConfigFile(self):
        self.WriteConfigFloat("longitude", self.longitude)
        self.WriteConfigFloat("latitude", self.latitude)
        self.WriteConfigString("city", self.city)
        self.FlushConfigFile()
        
def load(): return GfsWeekForecast()