from freemet.services.service import Service
import wx

_ = wx.GetTranslation

pages = ["MSG VIS", _("About")]
baseurl = "http://imkhp2.physik.uni-karlsruhe.de/~muehr/satpicsf/"
    
class ArchivePanel(wx.ScrolledWindow):
    def __init__(self, service):
        wx.ScrolledWindow.__init__(self, Service.notebook, -1)
        self.SetScrollRate(10, 10)
        
        self.service = service
        
        self.image = None
        self.sizer = None
        
        self.CreateLayout()
        
    def CreateLayout(self):
        if self.sizer:
            newimg = self.service.AddImageNotebookPage(self, "ukaarchive.jpg")
            self.sizer.Replace(self.image, newimg)
            self.image = newimg
        else:
            self.sizer = wx.BoxSizer(wx.VERTICAL)
            self.SetSizer(self.sizer)
            
            self.sizer.Add(self.__CreateArchiveControls())
        
            try:
                self.image = self.service.AddImageNotebookPage(self, "ukaarchive.jpg")
                self.sizer.Add(self.image)
            except:
                self.sizer.AddSpacer(1)
            
        self.sizer.Layout()
        self.Layout()
        
    def OnButton(self, evt):
        year = str(self.spinYear.GetValue())
        month = str(self.spinMonth.GetValue())
        day = str(self.spinDay.GetValue())
        hour = str(self.spinHour.GetValue())
        
        if len(month) == 1: month = "0" + month
        if len(day) == 1: day = "0" + day
        if len(hour) == 1: hour = "0" + hour
        
        if evt.GetId() == wx.ID_REFRESH:
            #url = baseurl + "DWD/%s_VIS/" % year
            #url += year + month + day + hour + "00.jpg"
            url = baseurl + "FV/EUR/"
            url += year + month + day + "1200_met8.jpg"

            self.service.url2file(url, "ukaarchive.jpg")
            self.CreateLayout()
        
    def __CreateArchiveControls(self):
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        # Date
        self.spinYear = wx.SpinCtrl(self, -1, size=(60, -1),
                                    min=2008, max=2008, initial=2008)
        self.spinMonth = wx.SpinCtrl(self, -1, size=(40, -1),
                                     min=1, max=12, initial=1)
        self.spinDay = wx.SpinCtrl(self, -1, size=(40, -1),
                                   min=1, max=31, initial=1)
        sizer.Add(self.spinYear)
        sizer.Add(self.spinMonth)
        sizer.Add(self.spinDay)
        
        # Hour
        self.spinHour = wx.SpinCtrl(self, -1, size=(40, -1),
                                    min=8, max=18, initial=0,
                                    style=wx.SP_WRAP | wx.SP_ARROW_KEYS)
        sizer.Add(self.spinHour)
        
        # Download button
        btn = wx.Button(self, wx.ID_REFRESH)
        self.Bind(wx.EVT_BUTTON, self.OnButton, btn)
        sizer.Add(btn)
        
        return sizer
    
class MsgVisDwdPanel(ArchivePanel):
    def __init__(self, service):
        ArchivePanel.__init__(self, service)

class UKaArchive(Service):
    def __init__(self):
        Service.__init__(self, "ukaarchive", _("Archive Uni Karlsruhe"), 0, 0,
                         pages, "maps")
        
    def Load(self, download, progressDlg=None, progressCount=0):
        return 0, False
    
    def CreatePage(self, index):
        if index == pages.index("About"):
            descr = _("Bernhard Muehr from the University of Karlsruhe (Germany)\n"\
                      "provides an archive with hundreds of satellite images\n"
                      "covering about one year.")
            p = self.CreateAboutPage(descr, "(c) 2008 Tobias Wulff", "GPL 2")
        
        elif index == pages.index("MSG VIS"):
            p = MsgVisDwdPanel(self)
            
        return p
    
def load(): return UKaArchive()