import wx
import wx.grid
import hashlib, re, urllib2
from freemet.services.service import Service

_ = wx.GetTranslation

baseurl = "http://www.wn.co.nz"
cameraSite = "/camera.php?cameraid="
cameraIds = [34, 12, 5, 8]

class WellingtonViews(Service):
    def __init__(self):
        Service.__init__(self, "wellingtonviews", _("Wellington Views"),
                         1, 5, [_("About")], "webcams")
        
        self.images = None
        self.pages = []
        
        self.SetNumDownloads(len(cameraIds))

    def Load(self, download, progressDlg=None, progressCount=0):
        self.pages = []
        self.images = []
        
        if not download: return 0, False
        
        def GetImgSrc(url):
            # Open camera website to get latest image source url
            #try:
            req = urllib2.Request(url)
            response = urllib2.urlopen(req)
            s = response.read()
            response.close()
            """
            except urllib2.URLError, e:
                print "Error downloading " + url
                return
            """
            
            # Find title
            reTitle = re.compile(":: (.*) Webcam ::")
            matchTitle = reTitle.search(s)
            imgtitle = matchTitle.groups()[0]
            
            # Find <img src> tag
            reImg = re.compile("img src=\"(.*jpg)\"")
            matchImg = reImg.search(s)
            imgsrc = matchImg.groups()[0]
            
            return imgsrc, imgtitle
            
        # Build download urls and filenames to download images to
        downloads = []
        for id in cameraIds:
            # Compose filename out of the camera id
            filename = "wgtnviews%i.jpg" % id
            
            cameraUrl, cameraTitle = GetImgSrc(baseurl + cameraSite + str(id))
            if not cameraUrl:
                print _("Image source for camera %i not found" % id)
                continue
            
            downloads.append((baseurl + cameraUrl, filename))
            self.pages.append(cameraTitle)

        # Now that all page titles are collected, tell service superclass
        # the notebook pages we need
        self.pages += [_("About")]
        self.SetPages(self.pages)
        
        return self.Download(download, downloads, progressDlg, progressCount)
    
    def CreatePage(self, index):
        if self.images == None: self.images = []
        if not self.pages: self.pages = [_("About")]
        
        if index == self.pages.index(_("About")):
            descr = _("This service downloads webcam images from the website\n"\
                      "Wellington Views (wn.co.nz).\n"\
                      "Add new webcams by appending your camera ids to the\n"\
                      "cameraIds list at the top of wellingtonviews.py.")
            p = self.CreateAboutPage(descr, "(c) 2008 Tobias Wulff", "GPL 2")
        
        else:
            p = wx.Panel(Service.notebook, -1)
            filename = "wgtnviews%i.jpg" % cameraIds[index]
            img = self.AddImageNotebookPage(p, filename)
            
            self.images.append(img)

        return p
    
def load(): return WellingtonViews()