from freemet.services.service import Service
import wx
import wx.animate
import time

_ = wx.GetTranslation

baseurl = "http://www.wetteronline.de/daten/radar/dwddg/"
file3h = "vie"
file90min = "std"
extension = ".gif"

pages = [_("Now"), _("3h"), _("90 min"), _("About")]

class Wetteronline(Service):
    def __init__(self):
        Service.__init__(self, "wetteronline", "Wetteronline", 3, 10, pages,
                         "satellite")
        self.animations = None
        
    def Load(self, download, progressDlg=None, progressCount=0):
        """
        Loads some images from wetteronline.de
        """
        
        curTime = time.time()
        t = time.gmtime()

        subMin = 0

        # Round down to the last quarter of the hour
        if t.tm_min >= 10 and t.tm_min < 25:    subMin = t.tm_min
        elif t.tm_min >= 25 and t.tm_min < 40:  subMin = t.tm_min - 15
        elif t.tm_min >= 40 and t.tm_min < 55:  subMin = t.tm_min - 30
        elif t.tm_min >= 55:                    subMin = t.tm_min - 45
        elif t.tm_min < 10:                     subMin = t.tm_min + 15
            
        t = time.gmtime(curTime - subMin * 60)
        
        dateUrl = time.strftime("%Y/%m/%d/", t)
        timeUrl = time.strftime("%H%M", t)
        
        downloads = [
            (baseurl + dateUrl + timeUrl + extension,
             "wetteronline.gif"),
            (baseurl + dateUrl + file3h + timeUrl + extension,
             "wetteronline180.gif"),
            (baseurl + dateUrl + file90min + timeUrl + extension,
             "wetteronline90.gif")]
        return self.Download(download, downloads, progressDlg, progressCount)
            
    def CreatePage(self, index):
        if self.animations == None: self.animations = []
            
        # Put image on panel "now"
        if index == 0:
            p = wx.Panel(Service.notebook, -1)
            ani = self.AddAnimatedImageNotebookPage(p, "wetteronline.gif")
            assert ani
            try:
                self.animations[0] = ani
            except:
                self.animations.append(ani)
        
        # Put image on panel "3h"
        elif index == 1:
            p = wx.Panel(Service.notebook, -1)
            ani = self.AddAnimatedImageNotebookPage(p, "wetteronline180.gif")
            assert ani
            try:
                self.animations[1] = ani
            except:
                self.animations.append(ani)
        
        # Put image on panel "90 min"
        elif index == 2:
            p = wx.Panel(Service.notebook, -1)
            ani = self.AddAnimatedImageNotebookPage(p, "wetteronline90.gif")
            assert ani
            try:
                self.animations[2] = ani
            except:
                self.animations.append(ani)
        
        # About page
        elif index == 3:
            descr = _("This service downloads the latest radar images from\n"\
                      "www.wetteronline.de, showing the current precipitation,\n"\
                      "and two animated sequences with the precipitation for\n"\
                      "the last 90 and 180 minutes.")
            p = self.CreateAboutPage(descr, "(c) 2008 Tobias Wulff", "GPL 2")
        
        return p
        
    def ActivateNotebookPage(self, index):
        if index >= len(self.animations): return
        self.animations[index].Play()
            
    def DeactivateNotebookPage(self, index):
        if index >= len(self.animations): return
        
        # Stop doesn't seem to work, animation on deactivated page
        # keeps running
        self.animations[index].Stop()
        
def load(): return Wetteronline()