import wx
import images

_ = wx.GetTranslation

class TaskBarIcon(wx.TaskBarIcon):
    TBMENU_RESTORE = wx.NewId()
    TBMENU_CLOSE   = wx.NewId()
    TBMENU_UPDATE  = wx.NewId()
    
    def __init__(self, frame):
        wx.TaskBarIcon.__init__(self)

        self.frame = frame
        
        # Bind mouse and menu events to functions in this class
        self.Bind(wx.EVT_TASKBAR_LEFT_DOWN, self.OnTaskBarLeftDown)
        self.Bind(wx.EVT_MENU, self.OnTaskBarActivate, id=self.TBMENU_RESTORE)
        self.Bind(wx.EVT_MENU, self.OnTaskBarClose, id=self.TBMENU_CLOSE)
        self.Bind(wx.EVT_MENU, self.OnTaskBarUpdate, id=self.TBMENU_UPDATE)
        
        # Load icon
        bmp = images.sonneicon.GetBitmap()
        icon = wx.IconFromBitmap(bmp)
        self.SetIcon(icon, "FreeMet")
        
    def ChangeIcon(self):
        img = images.sonneicon.GetImage()
        
        for x in range(13, 17):
            for y in range(6,26):
                img.SetRGB(x, y, 250, 80, 80)
                
        for x in range(8, 13):
            for y in range(12, 6, -1):
                img.SetRGB(x, y, 250, 80, 80)

        icon = wx.IconFromBitmap(img.ConvertToBitmap())
        self.SetIcon(icon, "FreeMet")
        
    def OnTaskBarLeftDown(self, evt):
        if self.frame.IsShown():
            self.frame.Hide()
            return
        
        if self.frame.IsIconized():
            self.frame.Iconize(False)
        if not self.frame.IsShown():
            self.frame.Show(True)
        self.frame.Raise()
        
    def OnTaskBarActivate(self, evt):
        if self.frame.IsIconized():
            self.frame.Iconize(False)
        if not self.frame.IsShown():
            self.frame.Show(True)
        self.frame.Raise()
    
    def OnTaskBarClose(self, evt):
        wx.CallAfter(self.frame.OnExit, None)
    
    def OnTaskBarUpdate(self, evt):
        self.frame.UpdateServices()
        
    def CreatePopupMenu(self):
        """
        Called by base class when an EVT_RIGHT_DOWN event is fired.
        """
        menu = wx.Menu()
        menu.Append(self.TBMENU_RESTORE, _("Restore"))
        menu.Append(self.TBMENU_CLOSE, _("Close"))
        menu.Append(self.TBMENU_UPDATE, _("Update services"))
        return menu