from threading import *
from os import getpid

class UpdateThread(Thread):
    """
    A thread that calls the update method of the given service after each
    interval seconds.
    """
    
    def __init__(self, frame, service):
        self.__name = service.GetNameTuple()[0]
        
        Thread.__init__(self, name=self.__name)
        
        self.__frame = frame
        self.__service = service
        self.__interval = service.GetUpdateInterval() * 60
        self.__isUpdating = False
        self.__pid = getpid()
        
        self.__updateEvent = Event()
        self.__exitEvent = Event()
        
    def GetPid(self):
        return self.__pid
    
    def Cancel(self):
        self.__exitEvent.set()
        self.__updateEvent.set()
        
    def Update(self):
        self.__updateEvent.set()

    def run(self):
        self.__pid = getpid()
        
        # Keep on updating till thread gets canceled/killed
        while not self.__exitEvent.isSet():
            # Reset timer
            if self.__interval == 0:
                # Wait for setting of update and exit events (method Cancel())
                self.__updateEvent.wait()
            else:
                # Wait for a given interval (in seconds) or for setting
                # of update and exit events (whatever occurs first)
                self.__updateEvent.wait(self.__interval)
                
            if self.__exitEvent.isSet(): continue
            
            # Update this thread's service
            self.__Update()
            
            self.__updateEvent.clear()
            
    def __Update(self):
        if self.__isUpdating: return
        self.__isUpdating = True
        
        # Put service into a queue to make the main thread set the status bar text 
        if self.__name not in self.__frame.pendingSetStatusText:
            self.__frame.pendingSetStatusText.append(self.__name)
        
        # Load service (i.e. download data) 
        self.__service.Load(True)
        
        # Check exit event before putting service to reload queue
        if self.__exitEvent.isSet(): return
        
        # Put service to reload queue
        if self.__name not in self.__frame.pendingReload:
            self.__frame.pendingReload.append(self.__name)
            
        self.__isUpdating = False
        